module Test where  -- -*- encoding: utf-8; -*- --

import Test.Tasty
import Test.Tasty.HUnit
import qualified Test.Tasty.QuickCheck as QC
import qualified Test.Tasty.SmallCheck as SC

import Prelude hiding (lookup)
import Data.Maybe
import Map
import PizzaLager

run = defaultMain $ testGroup "All tests" [mapTests, higherOrderTests, pizzaTests]

map1 :: Map Int String
map1 = insert 8 "Kathrin" $ insert 157 "Björn" $  insert 199 "Hans" $ insert 399 "Dirk" $ insert 3 "Jörg" $ insert 13 "Til" $ insert 12 "Chris" $ insert 14 "Kathy" empty 

list1 :: [(Int, String)]
list1 = [(8, "Kathrin"), (157, "Björn"), (199, "Hans"), (399, "Dirk"), (3, "Jörg"), (13, "Til"), (12, "Chris"), (14, "Kathy")]

map2 :: Map Int Int 
map2 = insert 4 49 $ insert 14 30 $ insert 4 9 $ insert 1 7 $ insert 5 5 empty

katalog :: Katalog
katalog = insert "Salami" (Artikel "Salami" "Eine Salami" 99) $ insert "Teig" (Artikel "Teig" "Pizzateig" 49) $ insert "Käse" (Artikel "Käse" "Pizzakäse zum Belegen" 29) $ insert "Tomaten" (Artikel "Tomaten" "Tomaten für die Pizzasoße" 19) $ insert "Thunfisch" (Artikel "Thunfisch" "Thunfisch zum Belegen" 89) $ insert "Ananas" (Artikel "Ananas" "Ananas zum Belegen" 79) $ insert "Schinken" (Artikel "Schinken" "Schinken zum Belegen" 59 ) empty

lager :: Lager
lager = insert "Salami" 39 $ insert "Teig" 200 $ insert "Käse" 99 $ insert "Tomaten" 100 $ insert "Thunfisch" 13 $ insert "Ananas" 0 $ insert "Schinken" 12 empty

lager' :: Lager
lager' = insert "Salami" 38 $ insert "Teig" 197 $ insert "Käse" 96 $ insert "Tomaten" 97 $ insert "Thunfisch" 12 $ insert "Ananas" 0 $ insert "Schinken" 12 empty

speisekarte :: Speisekarte
speisekarte = pizzaHinzufuegen "Salami Pizza" ["Salami", "Teig", "Käse", "Tomaten"] $ pizzaHinzufuegen "Thunfisch Pizza" ["Teig", "Tomaten", "Käse", "Thunfisch"] $ pizzaHinzufuegen "Pizza Margherita" ["Teig", "Tomaten", "Käse"] $ pizzaHinzufuegen "Pizza Hawaii" ["Teig", "Tomaten", "Käse", "Ananas", "Schinken"] empty 

mapTests = testGroup "Tests for empty, insert, ..."
    [ testCase "empty Test" $ empty @=? (Leaf :: Map Int Int)
    , testCase "insert empty" $ insert 10 "Test" empty @=? Branch Leaf 10 "Test" Leaf
    , testCase "lookup new element" $ lookup 10 (insert 10 "Lukas" map1) @=? Just "Lukas"
    , testCase "lookup not found" $ lookup 10 map1 @=? Nothing
    , testCase "size test" $ size map1 @=? 8
    , testCase "size after insert" $ size (insert 10 "Lukas" map1) @=? 9
    , testCase "fromList test" $ fromList list1 @=? map1
    , testCase "insert delete eq" $ map1 @=? insert 14 "Kathy" (delete 14 map1) 
    , testCase "fromList . toList = 1" $ fromList (toList map1) @=? map1
    ]

higherOrderTests = testGroup "Higher order functions" 
    [ testCase "fmap (reverse) test" $ (elems . fmap reverse) map1 @=? map reverse (elems map1)
    , testCase "fmap (+1) test" $ (elems . fmap (+1)) map2 @=? map (+1) (elems map2) 
    , testCase "foldr (sum) test" $ sum map2 @=? sum (elems map2) 
    , testCase "foldr (product) test" $ product map2 @=? product (elems map2)
    ]

pizzaTests = testGroup "Pizza Tests" 
    [ testCase "Pizza adding pizza" $ let sp' = pizzaHinzufuegen "Pizza Vielkäse" ["Teig", "Tomaten", "Käse", "Käse", "Käse"] speisekarte in Just (Pizza "Pizza Vielkäse" ["Teig", "Tomaten", "Käse", "Käse", "Käse"]) @=? lookup "Pizza Vielkäse" sp'
    , testCase "Pizza not found" $ Nothing @=? lookup "Pizza Vielkäse" speisekarte
    , testCase "preisZutaten" $ 215 @=? preisZutaten ["Teig", "Tomaten", "Käse", "Käse", "Thunfisch"] katalog
    , testCase "preisPizza" $ 196 @=? preisPizza (fromJust $ lookup "Salami Pizza" speisekarte) katalog
    , testCase "zutatenVorhanden (positive)" $ True @=? zutatenVorhanden lager (fromJust $ lookup "Salami Pizza" speisekarte)
    , testCase "zutatenVorhanden (negative)" $ False @=? zutatenVorhanden lager (fromJust $ lookup "Pizza Hawaii" speisekarte)
    , testCase "entnehmeZutat" $ Just 38 @=? lookup "Salami" (entnehmeZutat "Salami" lager)
    , testCase "bestellen (positive)" $ Just (lager', 479) @=? bestellen ["Salami Pizza", "Pizza Margherita", "Thunfisch Pizza"] speisekarte lager katalog
    , testCase "bestellen (negative)" $ Nothing @=? bestellen ["Salami Pizza", "Pizza Thunfisch", "Pizza Hawaii"] speisekarte lager katalog
    ]

